@@ -1,9 +1,9 @@
 /*
- * Copyright 2005-2012 Gentoo Foundation
+ * Copyright 2005-2024 Gentoo Foundation
  * Distributed under the terms of the GNU General Public License v2
  *
  * Copyright 2005-2012 Ned Ludd        - <solar@gentoo.org>
- * Copyright 2005-2012 Mike Frysinger  - <vapier@gentoo.org>
+ * Copyright 2005-2024 Mike Frysinger  - <vapier@gentoo.org>
  */
 
 const char argv0[] = "dumpelf";
@@ -40,9 +40,9 @@ static void dumpelf(const elfobj *elf, size_t file_cnt)
 		"\n"
 		"/*\n"
 		" * ELF dump of '%s'\n"
-		" *     %ji (0x%jX) bytes\n"
+		" *     %" PRIi64 " (0x%" PRIX64 ") bytes\n"
 		" */\n\n",
-		elf->filename, elf->len, elf->len);
+		elf->filename, (int64_t)elf->len, (uint64_t)elf->len);
 
 	/* setup the struct to namespace this elf */
 #define MAKE_STRUCT(B) \
@@ -114,7 +114,7 @@ static void dumpelf(const elfobj *elf, size_t file_cnt)
  break_out_shdr:
 	printf("},\n");
 
-	/* finish the namespace struct and start the abitrary ones */
+	/* finish the namespace struct and start the arbitrary ones */
 	printf("\n.dyns = dumpedelf_dyn_%zu,\n", file_cnt);
 	printf("};\n");
 
@@ -125,13 +125,17 @@ static void dumpelf(const elfobj *elf, size_t file_cnt)
 		if (elf->elf_class == ELFCLASS ## B) { \
 		const Elf ## B ## _Phdr *phdr = phdr_dynamic_void; \
 		const Elf ## B ## _Dyn *dyn = elf->vdata + EGET(phdr->p_offset); \
+		if ((void *)dyn >= elf->data_end - sizeof(*dyn)) { \
+			printf(" /* invalid dynamic tags ! */ "); \
+			goto break_out_dyn; \
+		} \
 		i = 0; \
 		do { \
+			dump_dyn(elf, dyn++, i++); \
 			if ((void *)dyn >= elf->data_end - sizeof(*dyn)) { \
 				printf(" /* invalid dynamic tags ! */ "); \
 				break; \
 			} \
-			dump_dyn(elf, dyn++, i++); \
 		} while (EGET(dyn->d_tag) != DT_NULL); \
 		}
 		DUMP_DYNS(32)
@@ -139,6 +143,7 @@ static void dumpelf(const elfobj *elf, size_t file_cnt)
 	} else {
 		printf(" /* no dynamic tags ! */ ");
 	}
+ break_out_dyn:
 	printf("};\n");
 }
 
@@ -227,7 +232,7 @@ static void dump_notes(const elfobj *elf, size_t B, const void *memory, const vo
 	}
 
 	printf("\n\t/%c note section dump:\n", '*');
-	for (i = 0; ndata < memory_end && !corrupt; ++i) {
+	for (i = 0; ndata < memory_end - sizeof(*note) && !corrupt; ++i) {
 		note = ndata;
 		namesz = EGET(note->n_namesz);
 		descsz = EGET(note->n_descsz);
@@ -248,7 +253,7 @@ static void dump_notes(const elfobj *elf, size_t B, const void *memory, const vo
 		printf("\t * Elf%zu_Nhdr note%zu = {\n", B, i);
 		printf("\t * \t.n_namesz = %u, (bytes)", namesz);
 		if (name)
-			printf(" [%s]", name);
+			printf(" [%.*s]", namesz, name);
 		printf("\n");
 		printf("\t * \t.n_descsz = %u, (bytes)", descsz);
 		if (desc) {
@@ -507,7 +512,6 @@ static void dump_dyn(const elfobj *elf, const void *dyn_void, size_t dyn_cnt)
 
 /* usage / invocation handling functions */
 #define PARSE_FLAGS "vhV"
-#define a_argument required_argument
 static struct option const long_opts[] = {
 	{"verbose",   no_argument, NULL, 'v'},
 	{"help",      no_argument, NULL, 'h'},
@@ -524,21 +528,16 @@ static const char * const opts_help[] = {
 /* display usage and exit */
 static void usage(int status)
 {
-	size_t i;
-	printf("* Dump internal ELF structure\n\n"
-	       "Usage: %s <file1> [file2 fileN ...]\n\n", argv0);
-	printf("Options:\n");
-	for (i = 0; long_opts[i].name; ++i)
-		if (long_opts[i].has_arg == no_argument)
-			printf("  -%c, --%-13s* %s\n", long_opts[i].val,
-			       long_opts[i].name, opts_help[i]);
-		else
-			printf("  -%c, --%-6s <arg> * %s\n", long_opts[i].val,
-			       long_opts[i].name, opts_help[i]);
-	exit(status);
+	pax_usage(
+		"Dump internal ELF structure",
+		"<file1> [file2 fileN ...]",
+		PARSE_FLAGS,
+		long_opts,
+		opts_help,
+		status);
 }
 
-/* parse command line arguments and preform needed actions */
+/* parse command line arguments and perform needed actions */
 static void parseargs(int argc, char *argv[])
 {
 	int flag;
@@ -580,8 +579,6 @@ static void parseargs(int argc, char *argv[])
 #if PAX_UTILS_LIBFUZZ
 int LLVMFuzzerInitialize(int *argc, char ***argv)
 {
-	(void)argc;
-	(void)argv;
 	(void)parseargs;
 	security_init(false);
 	return 0;
@@ -593,6 +590,7 @@ int LLVMFuzzerTestOneInput(const uint8_t *data, size_t size)
 	if (elf == NULL)
 		return 0;
 	dumpelf(elf, 0);
+	unreadelf(elf);
 	return 0;
 }
 #else
