@@ -1,9 +1,9 @@
 /*
- * Copyright 2003-2012 Gentoo Foundation
+ * Copyright 2003-2024 Gentoo Foundation
  * Distributed under the terms of the GNU General Public License v2
  *
  * Copyright 2005-2012 Ned Ludd        - <solar@gentoo.org>
- * Copyright 2005-2012 Mike Frysinger  - <vapier@gentoo.org>
+ * Copyright 2005-2024 Mike Frysinger  - <vapier@gentoo.org>
  */
 
 #include "paxinc.h"
@@ -584,7 +584,7 @@ free_elf_and_return:
 
 	/* check class and stuff */
 	if (!DO_WE_LIKE_ELF(elf->data)) {
-		warn("we no likey %s: {%s,%s,%s,%s}",
+		warn("unknown ELF settings: %s: {%s,%s,%s,%s}",
 		     filename,
 		     get_elfeitype(EI_CLASS, elf->data[EI_CLASS]),
 		     get_elfeitype(EI_DATA, elf->data[EI_DATA]),
@@ -620,6 +620,11 @@ free_elf_and_return:
 		char invalid; \
 		const Elf ## B ## _Ehdr *ehdr = EHDR ## B (elf->ehdr); \
 		Elf ## B ## _Off size; \
+		/* Need enough bytes for all of ehdr. */ \
+		if (elf->len < (off_t)sizeof(*ehdr)) { \
+			warn("%s: Incomplete ELF header", filename); \
+			goto free_elf_and_return; \
+		} \
 		/* verify program header */ \
 		invalid = 0; \
 		if (EGET(ehdr->e_phnum) <= 0) \
@@ -679,7 +684,7 @@ elfobj *_readelf_fd(const char *filename, int fd, size_t len, int read_only)
 
 	buffer = mmap(0, len, PROT_READ | (read_only ? 0 : PROT_WRITE), (read_only ? MAP_PRIVATE : MAP_SHARED), fd, 0);
 	if (buffer == MAP_FAILED) {
-		warn("mmap on '%s' of %li bytes failed :(", filename, (unsigned long)len);
+		warn("mmap on '%s' of %zu bytes failed :(", filename, len);
 		return NULL;
 	}
 
