@@ -1,9 +1,9 @@
 /*
- * Copyright 2003-2012 Gentoo Foundation
+ * Copyright 2003-2024 Gentoo Foundation
  * Distributed under the terms of the GNU General Public License v2
  *
  * Copyright 2003-2012 Ned Ludd        - <solar@gentoo.org>
- * Copyright 2004-2012 Mike Frysinger  - <vapier@gentoo.org>
+ * Copyright 2004-2024 Mike Frysinger  - <vapier@gentoo.org>
  */
 
 const char argv0[] = "scanelf";
@@ -315,9 +315,9 @@ static void scanelf_file_get_symtabs(elfobj *elf, const void **sym, const void *
 			Elf32_Word sym_idx; \
 			Elf32_Word chained; \
 			\
-			if (!VALID_RANGE(elf, offset, nbuckets * 4)) \
+			if (!VALID_RANGE(elf, hash_offset, nbuckets * (uint64_t)4)) \
 				goto corrupt_hash; \
-			if (!VALID_RANGE(elf, offset, nchains * 4)) \
+			if (!VALID_RANGE(elf, hash_offset, nchains * (uint64_t)4)) \
 				goto corrupt_hash; \
 			\
 			for (b = 0; b < nbuckets; ++b) { \
@@ -761,7 +761,7 @@ static const char *scanelf_file_textrels(elfobj *elf, char *found_textrels, char
 	if (symtab_void && elf->phdr)
 		SCANELF_ELF_SIZED(SHOW_TEXTRELS);
 	if (!*found_textrels)
-		warnf("ELF %s has TEXTREL markings but doesnt appear to have any real TEXTREL's !?", elf->filename);
+		warnf("ELF %s has TEXTREL markings but doesn't appear to have any real TEXTREL's !?", elf->filename);
 
 	return NULL;
 }
@@ -1154,7 +1154,7 @@ static const char *scanelf_file_soname(elfobj *elf, char *found_soname)
  *        STV group: STV_DEFAULT:p STV_INTERNAL:i STV_HIDDEN:h STV_PROTECTED:P
  *        SHN group: SHN_UNDEF:u SHN_ABS:a SHN_COMMON:c {defined}:d
  *    The "defined" value in the SHN group does not correspond to a SHN_xxx define.
- * You can search for multiple symbols at once by seperating with a comma (",").
+ * You can search for multiple symbols at once by separating with a comma (",").
  *
  * Some examples:
  *    ELFs with a weak function "foo":
@@ -1828,7 +1828,6 @@ static void scanelf_envpath(void)
 
 /* usage / invocation handling functions */ /* Free Flags: c d j u w G H J K P Q U W */
 #define PARSE_FLAGS "plRmyAXz:xetrnLibSs:k:gN:TaqvF:f:o:E:M:DIYO:ZCBhV"
-#define a_argument required_argument
 static struct option const long_opts[] = {
 	{"path",      no_argument, NULL, 'p'},
 	{"ldpath",    no_argument, NULL, 'l'},
@@ -1924,46 +1923,16 @@ static const char * const opts_help[] = {
 /* display usage and exit */
 static void usage(int status)
 {
-	const char a_arg[] = "<arg>";
-	size_t a_arg_len = strlen(a_arg) + 2;
-	size_t i;
-	int optlen;
-	printf("* Scan ELF binaries for stuff\n\n"
-	       "Usage: %s [options] <dir1/file1> [dir2 dirN file2 fileN ...]\n\n", argv0);
-	printf("Options: -[%s]\n", PARSE_FLAGS);
-
-	/* prescan the --long opt length to auto-align */
-	optlen = 0;
-	for (i = 0; long_opts[i].name; ++i) {
-		int l = strlen(long_opts[i].name);
-		if (long_opts[i].has_arg == a_argument)
-			l += a_arg_len;
-		optlen = max(l, optlen);
-	}
-
-	for (i = 0; long_opts[i].name; ++i) {
-		/* first output the short flag if it has one */
-		if (long_opts[i].val > '~')
-			printf("      ");
-		else
-			printf("  -%c, ", long_opts[i].val);
-
-		/* then the long flag */
-		if (long_opts[i].has_arg == no_argument)
-			printf("--%-*s", optlen, long_opts[i].name);
-		else
-			printf("--%s %s %*s", long_opts[i].name, a_arg,
-				(int)(optlen - strlen(long_opts[i].name) - a_arg_len), "");
-
-		/* finally the help text */
-		printf("* %s\n", opts_help[i]);
-	}
-
-	puts("\nFor more information, see the scanelf(1) manpage");
-	exit(status);
+	pax_usage(
+		"Scan ELF binaries for stuff",
+		"<dir1/file1> [dir2 dirN file2 fileN ...]",
+		PARSE_FLAGS,
+		long_opts,
+		opts_help,
+		status);
 }
 
-/* parse command line arguments and preform needed actions */
+/* parse command line arguments and perform needed actions */
 #define do_pax_state(option, flag) \
 	if (islower(option)) { \
 		flags &= ~PF_##flag; \
